#include "AlgorithmTaskManage.h"

AlgorithmTaskManage::AlgorithmTaskManage(int humanDetectionLen,int licensePlateLen,int faceLen ):semaphore(humanDetectionLen),
    plateSemaphore(licensePlateLen),faceSemaphore(faceLen){
    this->humanDetectionLen=humanDetectionLen;
    this->licensePlateLen=licensePlateLen;
    this->faceLen=faceLen;

}

void AlgorithmTaskManage::initHumanDetectionManage(const QString &modelPaths,
                                                   float carShapeConfidence,float &height_reference,int &uniformColor) {
    for (int i = 0; i < humanDetectionLen; ++i) {
        HumanDetection* human=new HumanDetection(modelPaths,carShapeConfidence);
        human->setHuManParameter(height_reference,uniformColor);
        humanDetections.emplace_back(human);
    }
}
void AlgorithmTaskManage::initLicensePlateManage(const QString &modelPaths,
                                                 float carConfidence){
    for (int i = 0; i < licensePlateLen; ++i) {
        LicensePlateRecognition* licensePlateRecognition=new LicensePlateRecognition(modelPaths,carConfidence);
        licensePlateRecognitions.emplace_back(licensePlateRecognition);
    }
    
}
void AlgorithmTaskManage::initFaceReconitionHandle(std::map<QString,QString>&maps,int numberFaces,float confidence){
    for (int i = 0; i < faceLen; ++i) {
        FaceReconitionHandle *faceReconitionHandle=  new FaceReconitionHandle();
        faceReconitionHandle->initSourceImageMap(maps,numberFaces,confidence);

        faceReconitionHandles.emplace_back(faceReconitionHandle);
    }
}

void AlgorithmTaskManage::modifyImageFeature(std::map<QString,QString>&maps,int numberFaces,float confidence,bool isNull){
    std::lock_guard<std::mutex> lock(mtxFace);
    for (FaceReconitionHandle* face : faceReconitionHandles) {
        face->setImageChanged(true);
        if(isNull){
            face->featureRemove();
        }else {
            face->initSourceImageMap(maps,numberFaces,confidence);
        }
    }
}

AlgorithmTaskManage::~AlgorithmTaskManage(){
    Common & instace= Common::getInstance();
    
    for (auto task:humanDetections) {
        instace.deleteObj(task);
    }
    for (auto plate:licensePlateRecognitions) {
        instace.deleteObj(plate);
    }
    for (auto face:faceReconitionHandles) {
        instace.deleteObj(face);
    }
}

void* AlgorithmTaskManage::schedulingAlgorithm(int scheType) {
    if (scheType == 0x01) {
        return schedulingAlgorithmTemplate(humanDetections, mtxHuman);
    } else if (scheType == 0x02) {
        return schedulingAlgorithmTemplate(licensePlateRecognitions, mtxLicense);
    } else if (scheType == 0x03) {
        return schedulingAlgorithmTemplate(faceReconitionHandles, mtxFace);
    } else {
        qInfo() << "参数错误";
        return nullptr;
    }
}

void AlgorithmTaskManage::executeFindDoesItExistEmployee(const cv::Mat &source,std::list<vides_data::faceRecognitionResult>&faces,QString sSn){
    faceSemaphore.acquire();
    ScopeSemaphoreExit guard([this]() {
        faceSemaphore.release();  // 释放信号量
    });
    auto* selectedFaceReconition = static_cast<FaceReconitionHandle*>(schedulingAlgorithm(0x03));
    if (selectedFaceReconition!=nullptr && !selectedFaceReconition->getImageChanged()) {
        selectedFaceReconition->setIsRunning(true);
        // 调用选定对象的doesItExistEmployee函数
        qInfo() << "人脸识别算法抢到===>sn"<<sSn<<selectedFaceReconition;
        selectedFaceReconition->doesItExistEmployee(source, faces);
    } else {
        qInfo() << "没有可用的LicensePlateRecognition对象可以调度";
        return ;
    }
}


void AlgorithmTaskManage::executeFindlicensePlateNumber(const cv::Mat &source, QString &lpNumber,vides_data::requestLicensePlate &plate,
                                                        qint64 currentTime,QString sSn){
    plateSemaphore.acquire();
    ScopeSemaphoreExit guard([this]() {
        plateSemaphore.release();  // 释放信号量
    });
    auto* selectedLicensePlate = static_cast<LicensePlateRecognition*>(schedulingAlgorithm(0x02));
    if (selectedLicensePlate!=nullptr) {
        selectedLicensePlate->setIsRunning(true);
        // 调用选定对象的findHuManCar函数
        qInfo() << "车牌调度算法抢到===>sn"<<sSn<<selectedLicensePlate;
        selectedLicensePlate->licensePlateNumber(source, lpNumber,plate, currentTime);
    } else {
        qInfo() << "没有可用的LicensePlateRecognition对象可以调度";
        return ;
    }
}

int AlgorithmTaskManage::executeFindHuManCar(const cv::Mat &source, int res,
                                             std::vector<vides_data::ParkingArea> &currentPlate,std::map<int,int>&resMap,QString sSn) {
    
    semaphore.acquire();
    ScopeSemaphoreExit guard([this]() {
        semaphore.release();  // 释放信号量
    });
    auto* selectedHumanDetection = static_cast<HumanDetection*>(schedulingAlgorithm(0x01));
    if (selectedHumanDetection!=nullptr) {
        selectedHumanDetection->setIsRunning(true);
        // 调用选定对象的findHuManCar函数
        qInfo() << "人形调度算法抢到===>sn"<<sSn<<"res"<<selectedHumanDetection;
        int detectionResult = selectedHumanDetection->findHuManCar(source, res,resMap, currentPlate);
        return detectionResult;
    } else {
        qInfo() << "没有可用的HumanDetection对象可以调度";
        return -2;
    }
}
